--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: IDLESLEEP
-- @author    Valentin Schmidt
-- @version   0.2
-- @requires  xtra("UIHelper")
-- @desc      Reduces "CPU-Hogging" by letting making projector sleep on idle
--****************************************************************************

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
  $ = me.$
  
  $.loadXtras(me.libdir, ["UIHelper.x32"], ["UiHelper PPC Xtra.xtra"])
  
  -- arbitrary defaults
  $[#SleepTimeForeground] = 25
  $[#SleepTimeBackground] = 50
  $[#SleepTime] = $[#SleepTimeForeground]
  
  me.idleTime(20) -- 50/sec
  
  res = $.include(me.libdir&"idle.ls")
    
  return me
end

----------------------------------------
-- Destructor called by LSW
-- @callback
----------------------------------------
on __destruct (me)
	m = member("idle.ls")
	if voidP(m) then return
  if m.type=#script then m.erase()
end

----------------------------------------
-- Set the sleep times (in milliseconds)
-- @param {integer} foreground
-- @param {integer} background
-- @param {integer} [now]
----------------------------------------
on setSleepTime (me, foreground, background, now)
  if voidP(now) then now = foreground
  me.$[#SleepTimeForeground] = foreground
  me.$[#SleepTimeBackground] = background
  me.$[#SleepTime] = now
end

----------------------------------------
-- Set the idle handler period (in milliseconds)
-- @param {integer} [ms] - in ms, not ticks!
----------------------------------------
on idleTime (me, ms)
  if voidP(ms) then return _movie.idleHandlerPeriod * 1000/60
  else _movie.idleHandlerPeriod = ms * 60 / 1000
end
