--!parent
--!encoding=utf-8

--****************************************************************************
-- @package   Qt OOP API
-- @file			Object Interface
-- @version  	0.2
-- @author   	Valentin Schmidt
--****************************************************************************

global $

property ancestor
property _children
property _listeners

----------------------------------------
-- @constructor
----------------------------------------
on new (me, parentID)
		
	me._children = []
	me._listeners = [:]
		
	return me
end

----------------------------------------
--
----------------------------------------
on destroy (me)
  return $.Qt._qx.callObject(me._id, #destroy)
end

----------------------------------------
-- @return {symbol}
----------------------------------------
on objectClass (me)
  return $.Qt._qx.callObject(me._id, #objectClass)
end

----------------------------------------
-- @return {string|VOID}
----------------------------------------
on objectName (me)
  return $.Qt._qx.callObject(me._id, #objectName)
end

--parentWidget () -> int widgetID|VOID
on parentWidget (me)
  return $.Qt._qx.callObject(me._id, #parentWidget)
end

----------------------------------------
--
----------------------------------------
on setObjectName (me, p1)
  $.Qt._qx.callObject(me._id, #setObjectName, p1)
end

----------------------------------------
-- SIGNALS
----------------------------------------

----------------------------------------
-- me.disconnect() -> Disconnect everything connected to object
-- me.disconnect(signal) -> Disconnect everything connected to object's <signal>
-- me.disconnect(signal, receiver) -> Disconnect <receiver> connected to object's <signal>
-- me.disconnect(signal, receiver, method)  -> Disconnect [<receiver>, <method>] connected to object's <signal>

-- @return {bool}
----------------------------------------
on connect (me, signal, receiver, method)
	if voidP(receiver) then receiver = _movie
		
	-- TEST
	if integerP(signal) then
		$.Qt._qx.setEventCallback(me._id, signal, method, receiver)
		return TRUE
	end if
	
	if voidP(method) then method = signal
	if voidP(me._listeners[signal]) then me._listeners[signal] = []
	
	c = [receiver, method] -- connection target
	
	-- dont allow SAME connection twice?
	if me._listeners[signal].getPos(c) then return FALSE
	
	me._listeners[signal].add(c)
	
	if count(me._listeners[signal])=1 then
		$.Qt._qx.setObjectCallback(me._id, signal, signal, me) -- connect
	end if
	
	return TRUE	
end

----------------------------------------
-- me.disconnect() -> Disconnect everything connected to object
-- me.disconnect(signal) -> Disconnect everything connected to object's <signal>
-- me.disconnect(signal, receiver) -> Disconnect <receiver> connected to object's <signal>
-- me.disconnect(signal, receiver, method)  -> Disconnect [<receiver>, <method>] connected to object's <signal>

-- @return {bool}
----------------------------------------
on disconnect (me, signal, receiver, method)

	-- Disconnect everything connected to object
	if voidP(signal) then
		cnt = count(me._listeners)
		repeat with i = 1 to cnt
			signal = me._listeners.getPropAt(i)
			--$.Qt._qx.setObjectCallback(me._id, signal) -- disconnect (TODO)
		end repeat
		me._listeners = [:]
		return TRUE -- only return true if anything was deleted?
	end if
	
	-- nothing connected to <signal>, so exit
	if voidP(me._listeners[signal]) then return FALSE
	if count(me._listeners[signal])=0 then return FALSE
	
	-- Disconnect everything connected to object's <signal>
	if voidP(receiver) then
		--$.Qt._qx.setObjectCallback(me._id, signal) -- disconnect (TODO)
		me._listeners[signal] = []
		return TRUE -- only return true if anything was deleted?
	end if
	
	-- Disconnect <receiver> connected to object's <signal>
	if voidP(method) then		
		cnt = count(me._listeners[signal])
		repeat with i = cnt down to 1
			if me._listeners[signal][1]=receiver then me._listeners[signal].deleteAt(i)
		end repeat
		if count(me._listeners[signal])=0 then
			--$.Qt._qx.setObjectCallback(me._id, signal) -- disconnect (TODO)
		end if
		return TRUE -- only return true if anything was deleted?
	end if
	
	-- Disconnect [<receiver>: <method>] connected to object's <signal>	
	c = [receiver, method] -- connection target
	res = me._listeners[signal].deleteOne(c)
	if count(me._listeners[signal])=0 then
		--$.Qt._qx.setObjectCallback(me._id, signal) -- disconnect (TODO)
	end if
	return res

end

----------------------------------------
-- Using 1 ms single-shot timeouts to create new threads,
-- so scripting errors in callbacks don't fail silently.
-- @callback
----------------------------------------
on _detach (me, signal, id,v1,v2)
	$.detacher.detach(#_call, me, [signal, id,v1,v2])
end

----------------------------------------
-- We are detached from original thread, now handle signal
-- @callback
----------------------------------------
--on _call (me, signal, id,v1,v2)
--	listeners = me._listeners[signal]
--	if not voidP(listeners) then
--		repeat with l in listeners
--			call(l[2],l[1], id,v1,v2)
--		end repeat
--	end if
--end

on _call (me, args, ms)
	--put "_call", args, ms
	listeners = me._listeners[args[1]]
	if not voidP(listeners) then
		repeat with l in listeners
			call(l[2],l[1], args[2],args[3],args[4],  ms)
		end repeat
	end if
end